(*
                   uStrings.pas
     (c) 2001-2002 Paul C. L. Willmott, VP9MU

 This program is free software; you can redistribute it
 and/or modify it under the terms of the GNU General
 Public License as published by the Free Software
 Foundation; either version 2 of the License, or at
 your option, any later version.

 This program is distributed in the hope that it will
 be useful, but WITHOUT ANY WARRANTY; without even the
 implied warranty of MERCHANTABILITY or FITNESS FOR A
 PARTICULAR PURPOSE.  See the GNU General Public
 License for more details.

 You should have received a copy of the GNU General
 Public License along with this program; if not, write
 to the Free Software Foundation, Inc., 59 Temple
 Place, Suite 330, Boston, MA  02111-1307  USA

            Contact : vp9mu@amsat.org
*)

(*
     Revision History
     ----------------

     Version 1.0.0 - June  3, 2001
     Version 1.0.1 - June 14, 2001 - DVal added
     Version 1.0.2 - June 18, 2002 - NoZDP added
*)

UNIT uStrings ;

{
    Public Interface
}

INTERFACE

FUNCTION Upper(ss:STRING):STRING ;
FUNCTION Left(ss:STRING;Len:BYTE):STRING ;
FUNCTION Right(ss:STRING;Len:BYTE):STRING ;
FUNCTION TrimL(ss:STRING):STRING ;
FUNCTION TrimR(ss:STRING):STRING ;
FUNCTION Trim(ss:STRING):STRING ;
PROCEDURE DecodeConfig(sLine:STRING;VAR sToken:String;VAR sValue:STRING) ;
FUNCTION WVal(sValue:STRING):WORD ;
FUNCTION BVal(sValue:STRING):BYTE ;
FUNCTION DVal(sValue:STRING):DOUBLE ;
FUNCTION StoC(SValue:STRING):CHAR ;
FUNCTION Strng(cc:CHAR;Len:BYTE):STRING ;
FUNCTION WtoS(wValue:WORD):STRING ;
FUNCTION WtopS(wValue:WORD;Len:BYTE):STRING ;
FUNCTION DStr(dValue:DOUBLE;dp:BYTE):STRING ;
FUNCTION DSep(dValue:DOUBLE;dp:BYTE):STRING ;
FUNCTION DSepNoZDP(dValue:DOUBLE):STRING ;
FUNCTION CheckReal(sValue:STRING):BOOLEAN ;
FUNCTION PadR(sValue:STRING;Len:INTEGER):STRING ;
FUNCTION PadL(sValue:STRING;Len:INTEGER):STRING ;
FUNCTION NoZDP(dValue:DOUBLE):STRING ;
FUNCTION LooseDP(dValue:DOUBLE):DOUBLE ;
FUNCTION ShortenString(sIn:STRING;lSpace:integer):STRING;
{
     Hidden Implementation
}

IMPLEMENTATION

uses

     SysUtils, StrUtils ;
{
     NOTE: Turbo Pascal strings are a fixed length block of
           256 characters. They can be accessed as an array
           of character, e.g. ss[3] accesses the 3rd character
           of the string ss. The length of the string is stored 
           at ss[0], thus the largest string can only be 255
           characters long. There is no string termination 
           character as in "C", ... you use the string length
           instead.
}

{
     Return parameter string forced to upper case.
}

FUNCTION Upper(ss:STRING):STRING ;
VAR
     Index : BYTE ;
BEGIN
     FOR Index := 1 TO Length(ss) DO BEGIN
         ss[Index] := UpCase(ss[Index]) ;
     END ;
     Upper := ss ;
END { Upper } ;

{
     Returns the left-most Len characters of the parameter
     string. If the parameter string length is less than
     Len, then it is returned unchanged.
}

FUNCTION Left ;
BEGIN
     IF (Length(ss)>Len) THEN BEGIN
         SetLength(ss,Len) ;
     END ;
     Left := ss ;
END { Left } ;

{
     Returns the right-most Len characters of the parameter
     string. If the parameter string length is less than
     Len, then it is returned unchanged.
}

FUNCTION Right ;
VAR
     LenAct : BYTE ;
BEGIN
     LenAct := Length(ss) ;
     IF (Len<LenAct) THEN BEGIN
         Delete(ss,1,LenAct-Len) ;
     END ;
     Right := ss ;
END { Right } ;

{
     Returns the parameter string, with all spaces to the left
     of the first non-space character removed.
}

FUNCTION TrimL ;
BEGIN
     WHILE (Length(ss)>0) AND (ss[1]=' ') DO BEGIN
         Delete(ss,1,1) ;
     END ;
     TrimL := ss ;
END { TrimL } ;

{
     Returns the parameter string, with all spaces to the right
     of the last non-space character removed.
}

FUNCTION TrimR ;
VAR
     Len : BYTE ;
BEGIN
     Len := Length(ss) ;
     WHILE (Len>0) AND (ss[Len]=' ') DO BEGIN
         Delete(ss,Len,1) ;
         Len := Length(ss) ;
     END ;
     TrimR := ss ;
END { TrimL } ;

{
     Returns the parameter string, with all spaces to the left
     of the first non-space character, and all spaces from the
     right of the last non-space character removed.
}

FUNCTION Trim ;
BEGIN
     Trim := TrimR(TrimL(ss)) ;
END { Trim } ;

{
     Converts the parameter string to a WORD type, if the
     parameter string is prefixed with # then the number
     is treated as a Hex value. No error checking is done!
}

FUNCTION WVal(sValue:STRING):WORD ;
VAR
     _WVAL : WORD    ;
     Index : WORD    ;
     Digit : BYTE    ;
     Code  : INTEGER ;
BEGIN
     _WVAL := $0000 ;
     IF (sValue[1]='#') THEN BEGIN
         Index := 2 ; { Skip # }
         WHILE (Index<=Length(sValue)) DO BEGIN
             _WVAL := _WVAL SHL 8 ;
             Digit := Ord(sValue[Index]) ;
             IF (sValue[Index] IN ['0'..'9']) THEN BEGIN
                 _WVAL := _WVAL + Digit - Ord('0') ;
             END
             ELSE IF (sValue[Index] IN ['A'..'F']) THEN BEGIN
                 _WVAL := _WVAL + Digit - Ord('A') ;
             END ;
             Dec(Index) ;
         END ;
     END
     ELSE BEGIN
         Val(sValue,_WVAL,Code) ;
     END ;
     WVal := _WVAL ;
END { WVal } ;

{
     Converts the parameter string to a BYTE type, if the
     parameter string is prefixed with # then the number
     is treated as a Hex value. No error checking is done!
}

FUNCTION BVal(sValue:STRING):BYTE ;
BEGIN
     BVal := Lo(WVal(sValue)) ;
END { BVal } ;

{
     Converts the parameter string to a DOUBLE type.
     No error checking is done!
}

FUNCTION DVal(sValue:STRING):DOUBLE ;
VAR
     _DVAL : DOUBLE  ;
     Code  : INTEGER ;
BEGIN
     Val(sValue,_DVAL,Code) ;
     DVal := _DVAL ;
END { DVal } ;

FUNCTION CheckReal(sValue:STRING):BOOLEAN ;
VAR
     _DVAL : DOUBLE  ;
     Code  : INTEGER ;
BEGIN
     Val(sValue,_DVAL,Code) ;
     CheckReal := Code = 0 ;
END { CheckReal } ;


{
     This routine is used to split a line of the form:

     TOKEN STRING = VALUE STRING ' THIS IS A COMMENT

     into its component parts: sToken, and sValue

     Another including and after the ' character is considered
     to be a comment as is deleted.

     Double spaces are removed and the returned values are
     trimmed and changed to upper case.

     Used for reading configuration files!
}

PROCEDURE DecodeConfig(sLine:STRING;VAR sToken:String;VAR sValue:STRING) ;
VAR
     Index : WORD ;
BEGIN

{
     Strip Comments !
}

     Index := Pos('''',sLine) ;
     WHILE (Index<>0) DO BEGIN
         sLine := Left(sLine,Pred(Index)) ;
         Index := Pos('''',sLine) ;
     END ;

{
     Single Space
}

     sLine := Trim(sLine) + ' ' ;
     Index := Pos('  ',sLine) ;
     WHILE (Index<>0) DO BEGIN
         Delete(sLine,Index,1) ;
         Index := Pos('  ',sLine) ;
     END ;

{
     Clear Outputs
}

     sToken := '' ;
     sValue := '' ;

{
     Split into components biased on =, and force to upper case!
}

     Index := Pos('=',sLine) ;

     IF (Index<>0) THEN BEGIN
         sToken := Upper(Trim(Left (sLine,Pred(Index)))) ;
         Delete(sLine,1,Index) ;
         sValue := Upper(Trim(sLine)) ;
     END
     ELSE BEGIN
         sToken := Upper(Trim(sLine)) ;
     END ;

END { DecodeConfig } ;

{
     This routine converts the first character of a string
     to the data type CHAR.
}

FUNCTION StoC ;
BEGIN
     StoC := sValue[1] ;
END { StoC } ;

{
     This routine returns a string of the character cc of length Len.
}

FUNCTION Strng ;
VAR
     ss : STRING ;
     ii : integer ;
BEGIN
     ss := '' ;
     FOR ii := 1 TO Len DO BEGIN
         ss := ss + cc ;
     END ;
     Strng := ss ;
END { Strng } ;

{
     This routine converts an unsigned number to a string
}

FUNCTION WtoS ;
VAR
     ss : STRING ;
BEGIN
     Str(wValue:5,ss) ;
     WtoS := Trim(ss) ;
END { WtoS } ;

{
     This routine converts an unsigned number to a string, padded with
     leading zeroes to length Len
}

FUNCTION WtopS ;
BEGIN
     WtopS := Right(Strng('0',Len)+WtoS(WValue),Len) ;
END { WtopS } ;

FUNCTION DStr ;
VAR
     ss : STRING ;
BEGIN
     Str(dValue:30:dp,ss) ;
     DStr := Trim(ss) ;
END { DStr } ;

FUNCTION DSep ;
VAR
     ss : STRING ;
     tt : STRING ;
     ii : integer ;
     ll : integer ;
BEGIN
     Str(dValue:30:dp,ss) ;
     ss := Trim(ss) ;
     ii := Pos('.',ss) ;
     if (ii=0) then begin
         ii := Length(ss) ;
         tt := '' ;
     end
     else begin
         tt := Right(ss,Length(ss)-ii+1) ;
     end ;
     ll := 0 ;
     while (0<ii) do begin
         tt := ss[ii] + tt ;
         Inc(ll) ;
         Dec(ii) ;
         if (ll mod 3 = 0) and (0<ii) then begin
             tt := ',' + tt ;
         end ;
     end ;
     DSep := tt ;
END { DSep } ;

FUNCTION PadR ;
BEGIN
     PadR := Left(sValue+Strng(' ',Len),Len) ;
END { PadR } ;

FUNCTION PadL ;
BEGIN
     PadL := Right(Strng(' ',Len)+sValue,Len) ;
END { PadR } ;

{
     This function converts a double to a string.
     Decimal places are added as necessary, but
     trailing zeroes and non-necessary decimal points
     are removed.

     e.g.

     2.5

     2

     etc.
}

FUNCTION NoZDP(dValue:DOUBLE):STRING ;
BEGIN
     Result := DStr(dValue,20) ;
     while (Right(Result,1)='0') do begin
         Result := Left(Result,Pred(Length(Result))) ;
     end ;
     if (Right(Result,1)='.') then begin
         Result := Left(Result,Pred(Length(Result))) ;
     end ;
END { NoZDP } ;

FUNCTION DSepNoZDP(dValue:DOUBLE):STRING ;
VAR
     ss : STRING ;
     tt : STRING ;
     ii : integer ;
     ll : integer ;
BEGIN
     Str(dValue:40:20,ss) ;
     ss := Trim(ss) ;
     ii := Pos('.',ss) ;
     if (ii=0) then begin
         ii := Length(ss) ;
         tt := '' ;
     end
     else begin
         tt := Right(ss,Length(ss)-ii+1) ;
         Dec(ii) ;
     end ;
     ll := 0 ;
     while (0<ii) do begin
         tt := ss[ii] + tt ;
         Inc(ll) ;
         Dec(ii) ;
         if (ll mod 3 = 0) and (0<ii) then begin
             tt := ',' + tt ;
         end ;
     end ;
     Result := tt ;
     while (Right(Result,1)='0') do begin
         Result := Left(Result,Pred(Length(Result))) ;
     end ;
     if (Right(Result,1)='.') then begin
         Result := Left(Result,Pred(Length(Result))) ;
     end ;
END { DSepNoZDP } ;

FUNCTION LooseDP(dValue:DOUBLE):DOUBLE ;
BEGIN
     Result := StrToFloat(DStr(dValue,0)) ;
END { LooseDP } ;

FUNCTION ShortenString(sIn:STRING;lSpace:INTEGER):STRING;
  //This function returns a string of a specified length without cutting off words
VAR
    sOut   : String;
    ii    : Integer;
    iLast : integer;
    sNext : String;
BEGIN
  ii := Pos(' ',sIn);
  sNext := Left(sIn, Pos(' ',sIn));
  sOut := '';
  if sNext = '' then begin
    sOut := sIn;
  end
  else begin
    while (sIn <> '') and (length(sOut + sNext) < lSpace) and (sNext <> '') do begin
      sOut := sOut + sNext;
      sIn := Right(sIn,length(sIn) - length(sNext));
      sNext := Left(sIn, pos(' ',sIn));
    end;
  end;

  ShortenString := sOut;

END {ShortenString};
END. { uStrings }

