(*
            DGrabber - ufrmDGrabber.pas
       (c) 2003 Paul C. L. Willmott, VP9MU

 This program is free software; you can redistribute it
 and/or modify it under the terms of the GNU General
 Public License as published by the Free Software
 Foundation; either version 2 of the License, or at
 your option, any later version.

 This program is distributed in the hope that it will
 be useful, but WITHOUT ANY WARRANTY; without even the
 implied warranty of MERCHANTABILITY or FITNESS FOR A
 PARTICULAR PURPOSE.  See the GNU General Public
 License for more details.

 You should have received a copy of the GNU General
 Public License along with this program; if not, write
 to the Free Software Foundation, Inc., 59 Temple
 Place, Suite 330, Boston, MA  02111-1307  USA

            Contact : vp9mu@amsat.org
*)

(*
     Revision History
     ----------------

     1.0.0      Created May 15, 2003 PCLW
     1.0.1      Display Bug fixed, read existing file added May 16, 2003 PCLW
     1.0.2      Reopen bug Fix May 22, 2003 PCLW

*)

unit ufrmDGrabber;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Menus;

type

  TBlock = packed record
                 cType : AnsiChar ;
                 case AnsiChar of
                 'A' : ( cFiller   : packed array[1..$1A5] of byte ;
                         iOrbitNum : word ;
                        ) ;
                 'D' : ( cSpace    : AnsiChar ;
                         cFileID1  : AnsiChar ;
                         cFileID2  : AnsiChar ;
                         iBlocks   : word ;
                         iSeqNum   : word ;
                        ) ;
                 'X' : ( cBlock    : packed array[1..511] of byte ;
                        ) ;
           end ;

  TfrmDGrabber = class(TForm)
    btnReset: TButton;
    imgDGrabber: TImage;
    tmrCheckRegistry: TTimer;
    Label1: TLabel;
    lblOrbitNumber: TLabel;
    Label3: TLabel;
    lblFileID: TLabel;
    Label5: TLabel;
    lblNumBlocks: TLabel;
    Label7: TLabel;
    lblBlocksToGo: TLabel;
    Label9: TLabel;
    lblLastBlock: TLabel;
    lblCommand: TLabel;
    btnCommand: TButton;
    lblFinished: TLabel;
    mnuMainMenu: TMainMenu;
    mnuFile: TMenuItem;
    mnuHelp: TMenuItem;
    mnuExit: TMenuItem;
    mnuAbout: TMenuItem;
    procedure btnResetClick(Sender: TObject);
    procedure tmrCheckRegistryTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnCommandClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mnuExitClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
  private
    { Private declarations }
    iStatus     : array[0..255] of integer ;
    bInCapture  : boolean ;
    iNumBlocks  : word ;
    iBlocksToGo : word ;
    iOrbitNum   : word ;
    sOutFile    : string ;
    iLastBlock  : word ;
    sOutDir     : string ;
    procedure DrawBlock(iNumber,iStatus:integer) ;
    procedure CheckRegistry(sKey:string) ;
    procedure GetBlock(sFilename:string) ;
  public
    { Public declarations }
  end;

var
  frmDGrabber: TfrmDGrabber;

implementation

{$R *.dfm}

uses

     Registry,
     uStrings,
     INIFiles, ufrmAbout ;

const

     kNotUsed = clBlack ;
     kMissing = clRed ;
     kGotIt   = clBlue ;

     kMaxBlocks = 255 ;

procedure TfrmDGrabber.btnResetClick(Sender: TObject);
var
     iBlock : integer ;
begin
     lblFinished.Visible := false ;
     bInCapture := false ;
     iLastBlock := 0 ;
     for iBlock := kMaxBlocks downto 0 do begin
         iStatus[iBlock] := kNotUsed ;
         DrawBlock(iBlock,iStatus[iBlock]) ;
     end ;
     btnCommand.Enabled := false ;
     lblLastBlock.Caption := '0' ;
     lblFileID.Caption := 'XX' ;
     lblNumBlocks.Caption := '0' ;
     lblBlocksToGo.Caption := '255' ;
end;

procedure TfrmDGrabber.CheckRegistry(sKey: string) ;

var

     pReg      : TRegistry ;
     sFileName : string    ;

begin
     sFilename := '' ;
     pReg := TRegistry.Create ;
     try
          pReg.RootKey := HKEY_CURRENT_USER ;
          if (pReg.OpenKey(sKey, True)) then begin
              sFilename := pReg.ReadString('Log File') ;
              pReg.WriteString('Log File','') ;
              pReg.CloseKey ;
          end ;
     finally
          pReg.Free ;
     end ;
     if (sFilename <> '') then begin
         GetBlock(sFilename) ;
     end ;
end;

procedure TfrmDGrabber.DrawBlock(iNumber,iStatus: integer);
var
     pRect : TRect ;
const
     kSize = 12 ;
begin
     pRect.Top    := (iLastBlock div 16) * kSize ;
     pRect.Left   := (iLastBlock mod 16) * kSize ;
     pRect.Bottom := (iLastBlock div 16 + 1) * kSize - 1 ;
     pRect.Right  := (iLastBlock mod 16 + 1) * kSize - 1 ;
     imgDGrabber.Canvas.Brush.Color := clBlack ;
     imgDGrabber.Canvas.FrameRect(pRect);
     pRect.Top    := (iNumber div 16) * kSize ;
     pRect.Left   := (iNumber mod 16) * kSize ;
     pRect.Bottom := (iNumber div 16 + 1) * kSize - 1 ;
     pRect.Right  := (iNumber mod 16 + 1) * kSize - 1 ;
     imgDGrabber.Canvas.Brush.Color := clYellow ;
     imgDGrabber.Canvas.FrameRect(pRect);
     imgDGrabber.Canvas.Brush.Color := iStatus ;
     imgDGrabber.Canvas.FloodFill((iNumber mod 16) * kSize + kSize div 2 ,(iNumber div 16) * kSize + kSize div 2,clYellow,fsBorder) ;
     iLastBlock := iNumber ;
end;

procedure TfrmDGrabber.GetBlock(sFilename: string);

var

     fInFile  : file of TBlock ;
     fOutFile : file of TBlock ;
     rBlock   : TBlock ;
     rTmp     : TBlock ;
     iBlock   : word ;
begin

     {$I-}
     AssignFile(fInFile,sFilename) ;
     FileMode := fmOpenRead ;
     Reset(fInFile) ;
     {$I+}

     if (IOResult = 0) then begin

         Seek(fInFile, FileSize(fInFile)-1) ;
         Read(fInFile,rBlock) ;

         if (Byte(rBlock.cType) and $80 = 0) then begin

             case rBlock.cType of
             'A' : begin
                        iOrbitNum := rBlock.iOrbitNum ;
                        lblOrbitNumber.Caption := wtops(iOrbitNum,5) ;
                   end ;
             'D' : begin

                        if (lblFileID.Caption<>rBlock.cFileID1+rBlock.cFileID2) then begin
                            btnResetClick(Self) ;
                        end ;

                        lblLastBlock.Caption := wtos(rBlock.iSeqNum) ;

                        if (not bInCapture) then begin

                            lblFileID.Caption := rBlock.cFileID1 + rBlock.cFileID2 ;
                            lblNumBlocks.Caption := wtos(rBlock.iBlocks) ;
                            lblBlocksToGo.Caption := lblNumBlocks.Caption ;

                            iNumBlocks := rBlock.iBlocks ;

                            for iBlock := kMaxBlocks downto 0 do begin
                                if (iBlock<iNumBlocks) then begin
                                    iStatus[iBlock] := kMissing ;
                                end
                                else begin
                                    iStatus[iBlock] := kNotUsed ;
                                end ;
                                DrawBlock(iBlock,iStatus[iBlock]) ;
                            end ;

                            iBlocksToGo := iNumBlocks ;

                            sOutFile := sOutDir + 'D' + wtops(iOrbitNum,5) + '_' + rBlock.cFileID1 + rBlock.cFileID2 + '.TLM' ;

                            {$I-}
                            AssignFile(fOutFile,sOutFile) ;
                            FileMode := fmOpenReadWrite ;
                            Reset(fOutFile) ;
                            {$I+}
                            if (IOResult = 0) then begin
                                while (not Eof(fOutFile)) do begin
                                    Read(fOutFile,rTmp) ;
                                    iStatus[rTmp.iSeqNum] := kGotIt ;
                                    DrawBlock(rTmp.iSeqNum,iStatus[rTmp.iSeqNum]) ;
                                    Dec(iBlocksToGo) ;
                                    lblBlocksToGo.Caption := wtos(iBlocksToGo) ;
                                    if (iBlocksToGo=0) then begin
                                        lblFinished.Visible := true ;
                                    end ;
                                end ;
                            end
                            else begin
                                ReWrite(fOutFile) ;
                            end ;
                            CloseFile(fOutFile) ;

                            bInCapture := true ;
                            btnCommand.Enabled := true ;

                        end ;

                        if (iStatus[rBlock.iSeqNum]=kMissing) then begin
                            {$I-}
                            AssignFile(fOutFile,sOutFile) ;
                            FileMode := fmOpenReadWrite ;
                            Reset(fOutFile) ;
                            {$I+}
                            if (IOResult = 0) then begin
                                Seek(fOutFile, FileSize(fOutFile)) ;
                                Write(fOutFile,rBlock) ;
                            end ;
                            CloseFile(fOutFile) ;
                            iStatus[rBlock.iSeqNum] := kGotIt ;
                            Dec(iBlocksToGo) ;
                            lblBlocksToGo.Caption := wtos(iBlocksToGo) ;
                            if (iBlocksToGo=0) then begin
                                lblFinished.Visible := true ;
                            end ;
                        end ;

                        DrawBlock(rBlock.iSeqNum,iStatus[rBlock.iSeqNum]) ;

                   end ;
             end ;
         end ;
     end ;
     CloseFile(fInFile) ;

end;

procedure TfrmDGrabber.tmrCheckRegistryTimer(Sender: TObject);
var
     sFilename : string ;

begin
     tmrCheckRegistry.Enabled := false ;
     CheckRegistry('\Software\VB and VBA Program Settings\P3TC\Command') ;
     CheckRegistry('\Software\AE4JY Software\AO40Rcv\AMSAT') ;
     tmrCheckRegistry.Enabled := true ;
end;

procedure TfrmDGrabber.FormCreate(Sender: TObject);
var
     pINIFile : TINIFile ;
begin
     pINIFile := TINIFile.Create('DGrabber.ini') ;
     try
         Top := pINIFile.ReadInteger('Position','Top',Top);
         Left := pINIFile.ReadInteger('Position','Left',Left);
         sOutDir := pINIFile.ReadString('Paths','D-Block Merge File','C:\');
     finally
         pINIFile.Free ;
     end ;

     btnResetClick(Self) ;
end;

procedure TfrmDGrabber.btnCommandClick(Sender: TObject);

var

     iMaxBlockStart : word ;
     iMaxBlockSize  : word ;
     iCurBlockStart : word ;
     iCurBlockSize  : word ;
     iCurIndex      : word ;

begin

     if (iBlocksToGo>0) then begin

         iMaxBlockStart := 0 ;
         iMaxBlockSize  := 0 ;

         for iCurBlockStart := 0 to iNumBlocks-1 do begin

             if (iStatus[iCurBlockStart]=kMissing) then begin

                 iCurBlockSize := 1 ;

                 iCurIndex := iCurBlockStart + 1 ;
                 if (iCurIndex=iNumBlocks) then begin
                     iCurIndex := 0 ;
                 end ;

                 while (iCurIndex<>iCurBlockStart) and
                       (iStatus[iCurIndex]=kMissing) do begin
                     Inc(iCurBlockSize) ;
                     Inc(iCurIndex) ;
                     if (iCurIndex=iNumBlocks) then begin
                         iCurIndex := 0 ;
                     end ;
                 end ;

                 if (iCurBlockSize>iMaxBlockSize) then begin
                     iMaxBlockStart := iCurBlockStart ;
                     iMaxBlockSize := iCurBlockSize ;
                 end ;

             end ; { if kMissing }

         end ; { for iCurStartBlock }

         if (iMaxBlockSize=1) then begin
             lblCommand.Caption := wtos(iMaxBlockStart) + ' (1 Block)' ;
         end
         else begin
             lblCommand.Caption := wtos(iMaxBlockStart) + ' ('+wtos(iMaxBlockSize) + ' Blocks)' ;
         end ;

     end ; { if (iBlocksToGo>0) }

end;

procedure TfrmDGrabber.FormClose(Sender: TObject;
  var Action: TCloseAction);

var
     pINIFile : TINIFile ;
begin
     pINIFile := TINIFile.Create('DGrabber.ini') ;
     try
         pINIFile.WriteInteger('Position','Top',Top);
         pINIFile.WriteInteger('Position','Left',Left);
         pINIFile.WriteString('Paths','D-Block Merge File',sOutDir);
         pINIFile.UpdateFile ; // Flush memory copy to disc
     finally
         pINIFile.Free ;
     end ;

     Action := caFree ;

end;

procedure TfrmDGrabber.mnuExitClick(Sender: TObject);
begin
     Self.Close ;
end;

procedure TfrmDGrabber.mnuAboutClick(Sender: TObject);
begin
     frmAbout := TfrmAbout.Create(nil) ;
     frmAbout.ShowModal ;
end;

end.
